package schedule

import (
	"context"
	"github.com/opsgenie/opsgenie-go-sdk-v2/client"
	"os"
)

type Client struct {
	client *client.OpsGenieClient
}

func NewClient(config *client.Config) (*Client, error) {
	opsgenieClient, err := client.NewOpsGenieClient(config)
	if err != nil {
		return nil, err
	}
	return &Client{opsgenieClient}, nil
}

func (c *Client) Create(context context.Context, request *CreateRequest) (*CreateResult, error) {
	result := &CreateResult{}
	err := c.client.Exec(context, request, result)
	if err != nil {
		return nil, err
	}
	return result, nil
}

func (c *Client) Get(context context.Context, request *GetRequest) (*GetResult, error) {
	result := &GetResult{}
	err := c.client.Exec(context, request, result)
	if err != nil {
		return nil, err
	}
	return result, nil
}

func (c *Client) Update(context context.Context, request *UpdateRequest) (*UpdateResult, error) {
	result := &UpdateResult{}
	err := c.client.Exec(context, request, result)
	if err != nil {
		return nil, err
	}
	return result, nil
}

func (c *Client) Delete(context context.Context, request *DeleteRequest) (*DeleteResult, error) {
	result := &DeleteResult{}
	err := c.client.Exec(context, request, result)
	if err != nil {
		return nil, err
	}
	return result, nil
}

func (c *Client) List(context context.Context, request *ListRequest) (*ListResult, error) {
	result := &ListResult{}
	err := c.client.Exec(context, request, result)
	if err != nil {
		return nil, err
	}
	return result, nil
}

func (c *Client) GetTimeline(context context.Context, request *GetTimelineRequest) (*TimelineResult, error) {
	result := &TimelineResult{}
	err := c.client.Exec(context, request, result)
	if err != nil {
		return nil, err
	}
	return result, nil
}

func (c *Client) ExportSchedule(context context.Context, request *ExportScheduleRequest) (*os.File, error) {
	result := &exportScheduleResult{}

	file, err := os.Create(request.ExportedFilePath + request.getFileName())
	if err != nil {
		return nil, err
	}

	defer file.Close()

	err = c.client.Exec(context, request, result)
	if err != nil {
		return nil, err
	}

	_, err = file.Write(result.FileContent)
	if err != nil {
		return nil, err
	}
	return file, nil
}
